//add long and lat voice by making a dual array for edges too and then adding up the lengths of the edges per long and lat. 
//like that we have less voices with more meaning!!

PrintWriter output;
PApplet sketch;
//add sound

boolean save = true; //save Frames toggle with 's'
//background stuff
PImage img; 

int nodeCount = int(pow(22, 2));  
int histLength = 25;
int count = 0 ; 
float radCore = 300.0; 
float radTubeX = 200.0; 
float radTubeY = 400.0; 
float depthOffset = 300.0; 
int nodeSquare = int(sqrt(nodeCount)); 
float impact = 80; 
float damp = 0.994;
float springConstant = 0.09;
float homeForceFactor = -0.06; 
float spin = 1.0/8.5*TWO_PI; 
float formant = 0.42; 
int lineCount = 0; 
float sRad = 15.0;


float step, offset; 

Node[][] nodeArray = new Node[nodeSquare][nodeSquare];
Edge[][] uEdgeArray = new Edge[nodeSquare][nodeSquare];
Edge[][] vEdgeArray = new Edge[nodeSquare][nodeSquare];
float[] uSum = new float[nodeSquare];
float[] vSum = new float[nodeSquare];

void setup() {
  println("nodeCount = "+nodeCount);
  sketch = this;
  img = loadImage("bg.jpg");
  frameRate(30); 
  size(1024, 1024, P3D);
  String timestamp = str(millis()); 
  String filename = "positions_"+timestamp; 
  output = createWriter(filename+".txt"); 
  hint(ENABLE_STROKE_PERSPECTIVE); 
  smooth(4); 
  sphereDetail(12); 
  step = height/nodeSquare; 
  //noCursor(); 
  init();
}

void init() {

  float theta = TWO_PI/nodeSquare; 
  for (int i = 0; i < nodeSquare; i++) {
    float cx = sin(theta*i);
    float cy = 0;
    float cz = cos(theta*i);
    for (int j = 0; j < nodeSquare; j++) {
      float currAngle = map(theta*j, 0, TWO_PI, -1.0, PI+1.0); 
      float xpos = radCore*cx + radTubeX*sin(currAngle)*cx;
      float ypos = radCore*cy + radTubeY*cos(currAngle);
      float zpos = radCore*cz + radTubeX*sin(currAngle)*cz;
      float jCenter = map(theta*j, 0, TWO_PI, -1.0, 1.15); 
      color c = color(255*jCenter*jCenter);
      nodeArray[i][j] = new Node(count, xpos, ypos, zpos, c);
    }
  }

  for (int k = 0; k < nodeSquare; k++) {
    for (int l = 0; l < nodeSquare; l++) {
      if (l < nodeSquare-1) {
        //this does the ladder
        float d = nodeArray[k][l].position.dist(nodeArray[k][l+1].position);
        uEdgeArray[k][l] = new Edge(nodeArray[k][l], nodeArray[k][l+1], d);
      }
      if (k < nodeSquare-1) {
        //this does the sprockets
        float d = nodeArray[k][l].position.dist(nodeArray[k+1][l].position);
        vEdgeArray[k][l] = new Edge(nodeArray[k][l], nodeArray[k+1][l], d);
      }
      if (k == nodeSquare-1) {
        float d = nodeArray[k][l].position.dist(nodeArray[0][l].position);
        vEdgeArray[k][l] = new Edge(nodeArray[k][l], nodeArray[0][l], d);
      }
    }
  }
}



void draw() {
  background(img); //background(0, 255, 0);
  pointLight(255, 255, 255, 0, 0, 0);
  ambientLight(128, 128, 128, 0, 0, 0);

  //camera
  float theta = frameCount/1000.0; //(frameCount%5000)*TWO_PI/5000.0; 
  float rad = 1400.0; 
  float xpos = sin(theta)*rad; 
  float ypos = (cos(theta)*rad); //0; 
  float zpos = 0;//(cos(theta)*rad); 
  camera(xpos, ypos, zpos, 0, 0, 0, 0, 1, 0);


  for (int i = 0; i < nodeSquare; i++) {
    for (int j = 0; j < nodeSquare; j++) {
      if (j < nodeSquare-1) {
        uEdgeArray[i][j].update();
        uEdgeArray[i][j].display();
      }
      vEdgeArray[i][j].update();
      vEdgeArray[i][j].display();
      nodeArray[i][j].update();
      nodeArray[i][j].shiftAdd(); 
      nodeArray[i][j].display();
    }
  }

  for (int p = 0; p < nodeSquare; p++) {
    uSum[p] = 0.0;
    vSum[p] = 0.0;
  }

  for (int m = 0; m < nodeSquare; m++) {
    for (int n = 0; n < nodeSquare; n++) {
      if (n < nodeSquare-1) {
        uSum[m] = uSum[m]+uEdgeArray[m][n].stretch;
      }
      vSum[m] = vSum[m]+vEdgeArray[m][n].stretch;
    }
  }

  if (save) {
    saveFrame("strike-#####.tga");
    //sonify
    for (int q = 0; q < nodeSquare; q++) {
      float uFreq = 440.0+uSum[q]; //(pow(stretch, 2.0)/2.5); 
      float vFreq = 440.0+vSum[q]; //(pow(stretch, 2.0)/2.5);
      uFreq = constrain(vFreq, 55, 15000);
      vFreq = constrain(vFreq, 55, 15000);
      float uPanFactor = map(q, 0, nodeSquare-1, -1.0, 1.0);
      float vPanFactor = 0.0;
      uPanFactor = constrain(uPanFactor, -1.0, 1.0);

      float uAmpFactor = map(pow(uFreq, 2), 0, 1000000, -0.99, 0.99);
      uAmpFactor = constrain(uAmpFactor, -0.99, 0.99); 
      float vAmpFactor = map(pow(vFreq, 2), 0, 1000000, -0.99, 0.99);
      vAmpFactor = constrain(vAmpFactor, -0.99, 0.99); 
      //printout
      output.println(lineCount+", "+uFreq+" "+uPanFactor+" "+uAmpFactor+";"); // Write the coordinate to the file
      lineCount++;
      output.println(lineCount+", "+vFreq+" "+vPanFactor+" "+vAmpFactor+";"); // Write the coordinate to the file
      lineCount++;
    }
  }
}

void clapper(float x, float y, float z) {
  for (int i = 0; i < nodeSquare; i++) {
    for (int j = 0; j < nodeSquare; j++) {
      nodeArray[i][j].clicked(x, y, z);
    }
  }
}


void keyTyped() {
  if (key == 'r') init();
  if (key == ' ') clapper(width/3, height/3, -10.0);
  if (key == 's') { 
    save = true;
    println("now saving!");
  } //toggle recording; 
  if (key == 'q') {
    save = false; //toggle recording;
    output.flush(); // Writes the remaining data to the file
    output.close(); // Finishes the file
    exit(); // Stops the program
  }
}

void mouseClicked() {
  float zdepth = 0; 
  clapper(mouseX-width/2, mouseY-height/2, 0);
  println("clicked at: "+mouseX+", "+mouseY+", "+zdepth);
}
